from .buttons import Button
from discord.http import Route

import discord

from typing import List

url = "https://discord.com/api/v8"


class V8Route(Route):
    BASE = "https://discord.com/api/v8"

def jsonifyMessage(content = None, *, tts=False,
                embed: discord.Embed = None, embeds: List[discord.Embed], nonce: int = None,
                allowed_mentions: discord.AllowedMentions = None, reference: discord.MessageReference = None, mention_author: bool = None, buttons: List[Button] = None):
    """Turns parameters from the `discord.TextChannel.send` function into json for requests"""
    payload = { "tts": tts }
    
    if content is not None:
        payload["content"] = content
    
    if nonce is not None:
        payload["nonce"] = nonce
    
    if embed is not None and embeds is not None:
        raise discord.InvalidArgument("cannot pass both 'embed' and 'embeds' parameter")

    if embed is not None:
        if type(embed) is not discord.Embed:
            raise TypeError("embed must be of type 'discord.Embed', not " + str(type(embed)))
        payload["embeds"] = [ embed.to_dict() ]
    if embeds is not None:
        if type(embeds) is not list:
            raise TypeError("embeds must be of type 'list', not " + str(type(embeds)))
        payload["embeds"] = [em.to_dict() for em in embeds]

    if reference is not None:
        if type(reference) not in [discord.MessageReference, discord.Message] and not issubclass(type(reference), discord.Message):
            raise TypeError("Reference must be of type 'discord.MessageReference' or 'discord.Message', not " + str(type(reference)))
        if type(reference) is discord.MessageReference:
            payload["message_reference"] = reference.to_dict()
        elif type(reference) is discord.Message:
            payload["message_reference"] = discord.MessageReference.from_message(reference).to_dict()

    if allowed_mentions is not None:
        payload["allowed_mentions"] = allowed_mentions.to_dict()
    if mention_author is not None:
        allowed_mentions = payload["allowed_mentions"] if "allowed_mentions" in payload else discord.AllowedMentions().to_dict()
        allowed_mentions['replied_user'] = mention_author
        payload["allowed_mentions"] = allowed_mentions

    if buttons:
        componentsJSON = {"components": []}
        
        wrappers: List[List[Button]] = []

        if len(buttons) > 1:
            curWrapper = []
            i = 0
            for _btn in buttons:
                btn: Button = _btn
                # i > 0         => Preventing empty component field when first button wants to newLine 
                if btn.new_line and i > 0:
                    wrappers.append(curWrapper)
                    curWrapper = [btn]
                    continue
                
                curWrapper.append(btn)
                i += 1
            wrappers.append(curWrapper)
        else:
            wrappers = [buttons]

        for wrap in wrappers:
            componentsJSON["components"].append({"type": 1, "components": [x.to_dict() for x in wrap]})

        payload |= componentsJSON 

    return payload


from typing import List
from .tools import jsonifyMessage, V8Route
from .receive import Message, getResponseMessage

import discord
from discord.ext import commands


class Buttons():
    """A button instance for using buttons
    
    Attributes
    ----------------
    send: `function`
        Sends a message to a `discord.TextChannel`
    
    """
    def __init__(self, client: commands.Bot):
        """
        This will create a new button intearction Listener

        For receiving the button event, scroll down to the bottom and take a look at the example

        Parameters
        ------------------
        ```py
        (commands.Bot) client
        ```
            The discord bot client

        Example
        ------------------
        Here's an example for using the button listener
        ```py
        # Your bot declaration should be here
        ...
        client.buttons = Buttons(client)

        @client.event("on_button_press")
        async def on_button(pressedButton, message):
            pass
        ```
        """
        self._discord = client
        self._discord.add_listener(self.on_socket_response)
    
    async def on_socket_response(self, msg):
        """Will be executed if the bot receives a socket response"""
        if msg["t"] != "INTERACTION_CREATE":
            return
        data = msg["d"]

        if data["type"] != 3:
            return
        
        guild = await self._discord.fetch_guild(data["guild_id"])
        user = discord.Member(data=data["member"], guild=guild, state=self._discord._connection)
        
        msg = await getResponseMessage(self._discord, data, user, True)

        self._discord.dispatch("button_press", msg.pressedButton, msg)
    

    async def send(self, channel: discord.TextChannel, content=None, *, tts=False,
            embed=None, embeds=None, file: discord.File=None, files: List[discord.File]=None, delete_after=None, nonce=None,
            allowed_mentions=None, reference=None, mention_author=None, buttons=None
        ) -> Message:
        """
        Sends a message to a `discord.TextChannel`

        Parameters
        -----------------
        ```py
        (discord.TextChannel) channel
        ```
            The channel where the message is going to be send
        ```py
        (str) content
        ```
            The message text content
        ```py
        (bool) tts
        ```
            Whether text-to-speech should be used
        ```py
        (discord.Embed) embed
        ```
            The embed in the message
        ```py
        (List[discord.Embed]) embeds
        ```
            A list of embeds for the message
        ```py
        (discord.File) file
        ```
            The attached file for the message
        ```py
        (List[discord.File]) file
        ```
            A list of Files which are going to be attached to the messaeg
        ```py
        (float) delete_after
        ```
            The numbers of seconds after which the message should be deleted
        ```py
        (int) nonce
        ```
            The nonce to use for sending this message
        ```py
        (List[discord.AllowedMentions])
        ```
            The mentions proceeded in the message
        ```py
        (discord.MessageReference or discord.Message) reference
        ```
            The message to refer to
        ```py
        (bool) mention_author
        ```
            Whether the author should be mentioned
        ```py
        (List[Button]) buttons
        ```
            A list of buttons included in the message


        Returns
        ---------------------
        ```py
        (Message)
        ```
            The sent message including buttons
        """
        if type(channel) != discord.TextChannel:
            raise discord.InvalidArgument("Channel must be of type discord.TextChannel")

        payload = jsonifyMessage(content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, reference=reference, mention_author=mention_author, buttons=buttons)
        

        route = V8Route("POST", f"/channels/{channel.id}/messages")
        
        r = await self._discord.http.request(route, json=payload)
        msg = await getResponseMessage(self._discord, r, response=False)
            
        if delete_after is not None:
            await msg.delete(delay=delete_after)
        
        return msg


import setuptools
from setuptools import version


def getVersion():
    with open("./discord_py_buttons/__init__.py", "r", encoding="utf-8") as f:
        return [f for f in f.readlines() if f.startswith("__version__")][0].split('"')[1].split('"')[0]
def getReadme():
    with open("./README.md", "r", encoding="utf-8") as f:
        return f.read()

setuptools.setup(
    name="discord-py-buttons",
    version=getVersion(),
    author="404kuso, RedstoneZockt, ",
    author_email="bellou9022@gmail.com, redstoneprofihd@gmail.com",
    description="A discord button handler for discord.py",
    long_description=getReadme(),
    long_description_content_type="text/markdown",
    url="https://github.com/KusoRedsto/discord_py_buttons",
    packages=setuptools.find_packages(),
    python_requires='>=3.9',
    classifiers=[
        "Programming Language :: Python :: 3"
    ]
)